package de.intevation.flys.exports;

import java.io.IOException;
import java.io.OutputStream;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import au.com.bytecode.opencsv.CSVWriter;

import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.WKms;

import de.intevation.flys.utils.Formatter;

/**
 * (CSV)Exporter for WDifferences.
 */
public class WDifferencesExporter extends AbstractExporter {

    /** The logger used in this exporter. */
    private static Logger logger = Logger.getLogger(WDifferencesExporter.class);


    public static final String WDIFF_CSV_KM_HEADER =
        "export.w_differences.csv.header.km";

    public static final String WDIFF_CSV_W_HEADER =
        "export.w_differences.csv.header.w";

    public static final String WDIFF_DEFAULT_CSV_KM_HEADER = "Fluss-Km";
    public static final String WDIFF_DEFAULT_CSV_W_HEADER  = "m";


    /** The storage that contains all WKms objects for the different facets. */
    protected List<WKms[]> data;


    public void init(Document request, OutputStream out, CallContext context) {
        logger.debug("WDifferencesExporter.init");

        super.init(request, out, context);

        this.data = new ArrayList<WKms[]>();
    }


    /**
     * Genereate data in csv format.
     */
    @Override
    public void generate()
    throws IOException
    {
        logger.debug("WDifferencesExporter.generate");

        if (facet == null) {
            throw new IOException("invalid (null) facet for exporter");
        }
        else if (facet.equals(AbstractExporter.FACET_CSV)) {
            generateCSV();
        }
        else {
            throw new IOException("invalid facet (" + facet + ") for exporter");
        }
    }


    /**
     * Adds given data.
     * @param d either a WKms or a CalculationResult to add to data.
     */
    @Override
    protected void addData(Object d) {
        if (d instanceof CalculationResult) {
            d = ((CalculationResult)d).getData();
            if (d instanceof WKms []) {
                data.add((WKms [])d);
            }
        }
        else if (d instanceof WKms) {
            data.add(new WKms[] { (WKms) d });
        }
    }


    /**
     * Lets writer write all data (including header).
     * @param writer Writer to write data with.
     */
    @Override
    protected void writeCSVData(CSVWriter writer) {
        logger.info("WDifferencesExporter.writeData");

        writeCSVHeader(writer);

        for (WKms[] tmp: data) {
            for (WKms wkms: tmp) {
                wKms2CSV(writer, wkms);
            }
        }
    }


    /**
     * Lets csvwriter write the header (first line in file).
     * @param write Writer to write header with.
     */
    protected void writeCSVHeader(CSVWriter writer) {
        logger.info("WDifferencesExporter.writeCSVHeader");

        writer.writeNext(new String[] {
            msg(WDIFF_CSV_KM_HEADER, WDIFF_DEFAULT_CSV_KM_HEADER),
            msg(WDIFF_CSV_W_HEADER, WDIFF_DEFAULT_CSV_W_HEADER)
        });
    }


    protected void wKms2CSV(CSVWriter writer, WKms wkms) {
        logger.debug("WDifferencesExporter.wQKms2CSV");

        NumberFormat kmf  = getKmFormatter();
        NumberFormat wf   = getWFormatter();
        int          size = wkms.size();

        for (int i = 0; i < size; i ++) {

            writer.writeNext(new String[] {
                kmf.format(wkms.getKm(i)),
                wf.format(wkms.getW(i))
            });
        }
    }


    /**
     * Returns the number formatter for kilometer values.
     *
     * @return the number formatter for kilometer values.
     */
    protected NumberFormat getKmFormatter() {
        return Formatter.getWaterlevelKM(context);
    }


    /**
     * Returns the number formatter for W values.
     *
     * @return the number formatter for W values.
     */
    protected NumberFormat getWFormatter() {
        return Formatter.getWaterlevelW(context);
    }


    @Override
    protected void writePDF(OutputStream out) {}
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
